; SEQUENTIAL COLLECTIONS

; VECTORS

; Vectors are an indexed sequential data structure, represented like:

[1 2 3]

; Elements can be retrieved by index:

(get [1 2 3] 0)
; 1
(get [1 2 3] 1)
; 2
(get [1 2 3] 10)
; nil <- result of calling with invalid index

; All Clojure collections can be counted
(count [1 2 3])
; 3

; Vectors can also be created like this:
(vector 1 2 3)
[1 2 3]

; Elements are added to the end with conj (short for conjoin)
(conj [1 2 3] 4 5 6)
[1 2 3 4 5 6]

; Clojure collections are immutable just like simple values. Any function that
; "changes" a collection actually just returns a new instance. E.g.:

(def v [1 2 3])
(conj v 4 5 6)
; [1 2 3 4 5 6]
v
; [1 2 3] <- but the original remains unchanged

; LISTS

; Lists are sequential linked lists that add new elements at the head of the
; list, instead of at the tail like vectors

; Constructing Lists

; Because lists are evaluated by invoking the first element as a function, we
; must quote a list to prevent evaluation:

(def cards '(10 :ace :jack 9))

; Lists are not indexed so they must be traversed using first and rest:

(first cards)
; 10
(rest cards)
; '(:ace :jack 9)

; Adding Elements

; conj can be used to add elements just as with vectors, but in this case the
; elements are added at the head (maintains constant time operation)

(conj cards :queen)
; (:queen 10 :ace :jack 9)

; Lists can also be used as a stack with peek and pop:

(def stack '(:a :b))
(peek stack)
; :a
(pop stack)
;(:b)
