; LITERALS

; Numeric Types
42        ; integer
-1.5      ; floating point
22/7      ; ratio

; Character Types
"hello"         ; string
\e              ; character
#"[0-9]+"       ; regular expression

; Symbols and Idents
map             ; symbol
+               ; symbol - most punctuation allowed
clojure.core/+  ; namespaced symbol
nil             ; null value
true false      ; booleans
:alpha          ; keyword
:release/alpha  ; keyword with namespace

; Literal Collections
'(1 2 3)     ; list
[1 2 3]      ; vector
#{1 2 3}     ; set
{:a 1, :b 2} ; map

; EVALUATION

; Source code is read as characters by the Reader either from .clj files or
; interactively via the REPL.

; The unit of source code is a Clojure expression, not a Clojure source file.
; Clojure source files are read as a series of expressions as if they had been
; typed manually at the REPL.

; Clojure is run on the JVM, and the compiler compiles the output from the
; Reader into bytecode that can be executed on the JVM.

; INTERACTIVITY

; The REPL has the following parts:
; 1. Read an expression (a string of characters) to produce Clojure data.
; 2. Evaluate the data returned from #1 to yield a result (also Clojure data).
; 3. Print the result by converting it from data back to characters.
; 4. Loop back to the beginning.

'(1 2 3)     ; Just a list of values, not code to evaluate
(1 2 3)      ; Will throw an error because trying to evaluate data as code

*1, *2, *3   ; Special symbols: the result of the nth previous evaluation

(require '[clojure.repl :refer :all]) ; Provides a number of special functions

(doc +)            ; Displays documentation for a function
(apropos "+")      ; Find functions that match a particular string or regex
(find-doc "word")  ; Like apropos but also searches docstrings
(dir clojure.repl) ; See full listing of functions in given namespace
(source demunge)   ; View source code for the given function

(def x 7) ; Setting definind x to 7 in the current namespace
(+ x x)   ; Using the above-defined variable, result will be 14

; Printing

(println "hi") ; Human readable printout with newline
(print "hi")   ; Human readable printout without newline
(prn "hi")     ; Printed as data that can read by the Reader with newline
(pr "hi")      ; Printed as data that can read by the Reader without newline

; TEST YOUR KNOWLEDGE

; 1. Using the REPL, compute the sum of 7654 and 1234.

(+ 7654 1234)

; 2. Rewrite the following algebraic expression as a Clojure expression:
; ( 7 + 3 * 4 + 5 ) / 10.

(/ (+ 7 (* 3 4) 5) 10)

; 3. Using REPL documentation functions, find the documentation for the rem and
; mod functions. Compare the results of the provided expressions based on the
; documentation.

(doc rem)
(doc mod)

; These are subtly different in that they behave differently when the numerator
; and divisor are of different signs.
; https://www.nickivance.com/writing/20181129_remainders/index.html
; https://math.stackexchange.com/questions/2179579/how-can-i-find-a-mod-with-negative-number

; 4. Using find-doc, find the function that prints the stack trace of the most recent REPL exception.

(find-doc "trace")

; Answer is clojure.repl/pst
